angular.module('core.csrfRequestService',
    ['core.baseUrlService', 'core.csrfStrongholdService', 'core.responseHandlerService','core.credStorageService'])

    .factory('csrfRequestService',
    ['$http', '$rootScope', 'baseUrlService', 'csrfStrongholdService', 'responseHandlerService', '$q','credStorageService',
        function ($http, $rootScope, baseUrlService, csrfStrongholdService, responseHandlerService, $q,credStorageService) {
            return {

                runHttpPromise: function(httpPromise, messageSuccessHandler, messageErrorHandler) {
                    $rootScope.loading = true;

                    var deferred = $q.defer();

                    httpPromise.then(function () {
                        responseHandlerService.handleResponse(arguments[0], messageSuccessHandler);
                        csrfStrongholdService.handleResponse(arguments[0]);
                        deferred.resolve(arguments[0].data);
                        $rootScope.loading = false;
                    }, function () {
                        responseHandlerService.handleResponse(arguments[0], messageErrorHandler);
                        csrfStrongholdService.handleResponse(arguments[0]);
                        deferred.reject(arguments[0].data);
                        $rootScope.loading = false;
                    });

                    return deferred.promise;
                },

                get: function (url, params, messageSuccessHandler, messageErrorHandler) {
                    console.log("---- GET " + url);
                    console.log(params);
                    console.log(csrfStrongholdService.getHeaders().headers);

                    return this.runHttpPromise($http({
                        "method": "GET",
                        "url": baseUrlService.getBaseUrl() + url,
                        "params": params,
                        "headers": csrfStrongholdService.getHeaders().headers
                    }), messageSuccessHandler, messageErrorHandler);
                },

                //  FIXME Amine : Faut il pas suppimer l'appel options ?
                options: function (url, params, messageSuccessHandler, messageErrorHandler) {
                    console.log("---- OPTIONS " + url);
                    console.log(params);
                    console.log(csrfStrongholdService.getHeaders().headers);

                    return this.runHttpPromise($http({
                        "method": "OPTIONS",
                        "url": baseUrlService.getBaseUrl() + url,
                        "params": params,
                        "headers": csrfStrongholdService.getHeaders().headers
                    }), messageSuccessHandler, messageErrorHandler);
                },
                post: function (url, model, messageSuccessHandler, messageErrorHandler) {
                    console.log("---- POST " + url);
                    console.log(model);
                    if(model!= null) {
                        credStorageService.setPassword(model.password);
                        credStorageService.setUserName(model.username);
                    }

                    console.log(csrfStrongholdService.getHeaders().headers);

                    return this.runHttpPromise($http.post(baseUrlService.getBaseUrl() + url,
                        model,
                        csrfStrongholdService.getHeaders()), messageSuccessHandler, messageErrorHandler);
                },
                delete: function (url, model, messageSuccessHandler, messageErrorHandler) {
                    console.log("---- DELETE " + url);
                    console.log(model);
                    console.log(csrfStrongholdService.getHeaders().headers);

                    //FIXME Amine : IL faut prevoir un nettoyage des com'z ainsi que les certains log

                    return this.runHttpPromise($http({
                        "method": "DELETE",
                        "url": baseUrlService.getBaseUrl() + url,
                        "data": model,
                        "headers": csrfStrongholdService.getHeaders().headers
                    }), messageSuccessHandler, messageErrorHandler);
                },
                put: function (url, params, model, messageSuccessHandler, messageErrorHandler) {
                    console.log("---- PUT " + url);
                    console.log(model);
                    console.log(params);
                    console.log(csrfStrongholdService.getHeaders().headers);

                    return this.runHttpPromise($http({
                        "method": "PUT",
                        "url": baseUrlService.getBaseUrl() + url,
                        "params": params,
                        "data": model,
                        "headers": csrfStrongholdService.getHeaders().headers,
                    }), messageSuccessHandler, messageErrorHandler);
                },
                postFile: function (url, mimeType, filePath, messageSuccessHandler, messageErrorHandler) {

                    $rootScope.loading = true;
                    var deferred = $q.defer();

                    console.log("---- POST FILE " + url);
                    console.log(mimeType);
                    console.log(filePath);
                    console.log(csrfStrongholdService.getMultipartHeaders()["headers"]);
                    
                    try {
                        var uri = encodeURI(baseUrlService.getBaseUrl() + url);

                        var options = new FileUploadOptions();
                        options.fileKey = "file";
                        options.fileName = "file";
                        options.mimeType = mimeType;
                        options.headers = csrfStrongholdService.getMultipartHeaders()["headers"];

                        var ft = new FileTransfer();
                        ft.upload(filePath, uri,
                        function(response) {
                            var data = {
                                "status" : response.responseCode,
                                "data" : JSON.parse(response.response)
                            };
                            responseHandlerService.handleResponse(data, messageSuccessHandler);

                            deferred.resolve(data.data);
                            $rootScope.loading = false;
                        },
                        function(response) {
                            var data = {
                                "status" : response.responseCode,
                                "data" : response.response
                            };
                            // alert(JSON.stringify(response));
                            responseHandlerService.handleResponse(data, messageErrorHandler);

                            deferred.reject(data.data);
                            $rootScope.loading = false;

                        }, options);
                    } catch (error) {
                        $rootScope.loading = false;
                        alert(JSON.stringify(error));
                    }
                    return deferred.promise;
                }
            };
        }]);
